import os
import os.path
import dircache

class Completor:
	needs_exact = 0
	extra_matches = []

	def set_needs_exact(self, val):
		self.needs_exact = val

	def set_extra_matches(self, val):
		if (type(val) == str):
			val = [val]
		self.extra_matches = val

	def possibilities(self, text):
		return []

	def match(self, text):
		ls = self.possibilities(text)
		return (text in ls)

	def completion(self, text):
		ls = self.possibilities(text)
		if (len(ls) == 0):
			return (text, 0)
		st = commonprefix(ls)
		flag = (st in ls)
		return (st, flag)

class FixedSetCompletor(Completor):
	def __init__(self, arr):
		self.array = arr

	def possibilities(self, text):
		ls = filter(
			lambda el: el.startswith(text),
			self.array)
		return ls

	def match(self, text):
		return (text in self.array)

class FilenameCompletor(Completor):	
	def possibilities(self, text):
		(dir, fil) = os.path.split(text)
		if (dir == ''):
			dir = os.getcwd()
		dir = os.path.expanduser(dir)
		dirls = dircache.listdir(dir)
		dirls = dirls[:]
		dircache.annotate(dir, dirls)
		ls = filter(
			lambda el: el.startswith(fil),
			dirls)
		return ls

	def match(self, text):
		(dir, fil) = os.path.split(text)
		if (dir == ''):
			dir = os.getcwd()
		dir = os.path.expanduser(dir)
		ls = dircache.listdir(dir)
		ls = ls[:]
		dircache.annotate(dir, ls)
		return (fil in ls)

	def completion(self, text):
		(dir, fil) = os.path.split(text)
		ls = self.possibilities(text)
		if (len(ls) == 0):
			return (text, 0)
		st = commonprefix(ls)
		flag = (st in ls)
		return (os.path.join(dir, st), flag)

def commonprefix(arr, start=0):
	if (len(arr) == 0):
		raise ValueError, 'commonprefix(): argument must be nonempty list'
	if (len(arr) == 1):
		return arr[0]
	s1 = min(arr)
	s2 = max(arr)
	end = min(len(s1), len(s2))
	ix = 0
	for ix in range(end):
		if (s1[ix] != s2[ix]):
			return s1[:ix]
	return s1[:end]
		
import log
