import string
import curses

statusline = None
statuswin = None
scrwidth = None
message = None

class StatusLine:
	def __init__(self):
		self.dirty = 0
		self.watch = watcher.Watchable(self)
		self.watch.link(self, self.see_change_status)
		self.miniscroll = 0
		self.miniimage = None
		self.miniimagepos = 0

	def see_change_status(self, obj, sig, detail):
		if (not minibuf.active):
			self.dirty = 1

	def see_change_minibuf(self, obj, sig, detail):
		self.miniimage = None
		if (minibuf.active):
			self.dirty = 1

def init(stdscr):
	global message, statusline

	statusline = StatusLine()
	message = ''
	resize(stdscr)

def redrawwin(stdscr):
	resize(stdscr)
	statuswin.redrawwin()
	
def resize(stdscr):
	global statuswin, scrwidth

	(hgt, wid) = stdscr.getmaxyx()
	scrwidth = wid
	
	statuswin = None
	statuswin = curses.newwin(1, scrwidth, hgt-1, 0)
	redraw()

def getkey():
	statuswin.keypad(1)
	if (minibuf.active and window.focuswin < 0):
		xpos = 1+len(minibuf.buf.prompt)+statusline.miniimagepos
		xpos = min(xpos, scrwidth-1)
	else:
		xpos = scrwidth-1
	return statuswin.getch(0, xpos)
	
def post(st):
	global message

	message = st
	statusline.watch.signal(watcher.SIG_DATA)
	

def refresh():
	if (statusline.dirty):
		statusline.dirty = 0
		statuswin.erase()
		redraw()
		statuswin.noutrefresh()

def redraw():
	if (window.focuswin < 0):
		statuswin.addstr(0, 0, '>')
	if (not minibuf.active):
		statuswin.addnstr(0, 1, message, scrwidth-2)
	else:
		width = (scrwidth-1)-(1+len(minibuf.buf.prompt))
		if (statusline.miniimage == None):
			lin = minibuf.buf.body
			linar = map(text.displaychar.__getitem__, map(ord, lin))
			im = string.join(linar, '')
			if (len(im) <= width):
				statusline.miniscroll = 0
				statusline.miniimage = im
				im1 = string.join(linar[ : minibuf.buf.mark], '')
				statusline.miniimagepos = len(im1)
			else:
				if (statusline.miniscroll > minibuf.buf.mark):
					statusline.miniscroll = minibuf.buf.mark - (width//2)
					statusline.miniscroll = max(0, statusline.miniscroll)
				im1 = string.join(linar[statusline.miniscroll : minibuf.buf.mark], '')
				if (len(im1) >= width):
					statusline.miniscroll = minibuf.buf.mark - (width//2)
					statusline.miniscroll = max(0, statusline.miniscroll)
					im1 = string.join(linar[statusline.miniscroll : minibuf.buf.mark], '')
				im2 = string.join(linar[minibuf.buf.mark : ], '')
				statusline.miniimage = im1+im2
				statusline.miniimagepos = len(im1)

		st = minibuf.buf.prompt + statusline.miniimage
		statuswin.addnstr(0, 1, st, scrwidth-2)

def clearforquit():
	statuswin.erase()
	statuswin.refresh()

# late imports
import log
import text
import watcher
import minibuf
import window

