import curses

import focus

defaultbufferclass = None

windows = None
scrheight = None
scrwidth = None

focuswin = 0

def setdefaultbuffer(bufclass):
	global defaultbufferclass
	assert issubclass(bufclass, buffer.Buffer)
	defaultbufferclass = bufclass

def getdefaultbuffer():
	if (defaultbufferclass == None):
		return buffer.Buffer
	assert issubclass(defaultbufferclass, buffer.Buffer)
	return defaultbufferclass
		
class Window(focus.Focusable):
	data = None
	body = None
	mode = None
	vpos = None
	height = None
	modedirty = 0
	debug = 0 ###

	def getcommandcontexts(self):
		return [Window] + focus.Focusable.getcommandcontexts(self)

	def setdata(self, dat):
		assert (dat == None or isinstance(dat, buffer.Buffer))
		if (self.data == dat):
			return
			
		if (self.data):
			self.data.watch.unlink(self)

		self.data = dat

		if (self.data):
			self.data.watch.link(self, self.see_change)
			self.data.watch.link(self, self.see_change_modeline)
		### signal change, or at least mark self for change

	def resize(self, vpos, hgt):
		self.vpos = vpos
		self.height = hgt

		self.body = None
		self.mode = None

		if (self.height < 2):
			return

		self.mode = curses.newwin(1, scrwidth, self.vpos+self.height-1, 0)
		self.body = curses.newwin(self.height-1, scrwidth, self.vpos, 0)

		self.mode.bkgdset('-', curses.A_REVERSE)
		self.mode.erase()
		self.modedirty = 1

		self.reinit()

	def reinit(self):
		pass

	def refresh(self):
		return
		
	def refresh_mode(self):
		if (not self.modedirty):
			return

		self.modedirty = 0
		self.mode.erase()
		if (focuswin >= 0 and windows[focuswin] == self):
			focusst = '>'
		else:
			focusst = ' '
		if (self.data.filename == None):
			dirtyst = '::'
		elif (self.data.isdirty):
			dirtyst = '**'
		else:
			dirtyst = '--'
		self.mode.addstr(0, 0, focusst+' '+dirtyst+' '+self.data.name+' '+self.modeinfo()) ###

	def see_change_modeline(self, obj, sig, detail):
		if (sig == watcher.SIG_MODE):
			self.modedirty = 1

	def modeinfo(self):
		return ''
		
	def see_change(self, obj, sig, detail):
		pass

	def checkvalid(self):
		if not(self in windows):
			raise command.StatusBeepWarning('Window no longer present')

class BlankWindow(Window):

	def reinit(self):
		self.body.bkgdset('.')
		self.body.erase()

def init(stdscr):
	global windows
	global scrwidth, scrheight

	(scrheight, scrwidth) = stdscr.getmaxyx()
	windows = []
	
	###
	bcla = getdefaultbuffer()
	buf = bcla('*scratch*')
	cla = buf.getviewclass()
	win = cla()
	win.setdata(buf)
	windows.append(win)

	recompute_sizes()

def redrawwin(stdscr):
	resize(stdscr)
	for win in windows:
		if (win.mode):
			win.mode.redrawwin()
		if (win.body):
			win.body.redrawwin()
	
def resize(stdscr):
	global scrwidth, scrheight

	(scrheight, scrwidth) = stdscr.getmaxyx()
	recompute_sizes()

def recompute_sizes():
	global focuswin
	
	while (len(windows) > 0 and len(windows) > (scrheight-1)//3):
		windows[0].setdata(None)
		windows.pop(0)

	if (focuswin >= len(windows)):
		focuswin = len(windows)-1
		
	count = len(windows)
	prevpos = 0
	for ix in range(count):
		win = windows[ix]
		if (ix == count-1):
			newpos = scrheight-1
		else:
			newpos = ((scrheight-1)*(ix+1)) // count
		win.resize(prevpos, newpos-prevpos)
		prevpos = newpos

def refresh():
	for win in windows:
		if (win.body):
			win.refresh()
		if (win.mode):
			win.refresh_mode()
		if (win.body):
			win.body.noutrefresh()
		if (win.mode):
			win.mode.noutrefresh()

def findonebybuf(buf):
	assert (isinstance(buf, buffer.Buffer))
	ls = filter(
		(lambda win: (win.data == buf)),
		windows)
	if (len(ls) == 0):
		return None
	return ls[0]

def findallbybuf(buf):
	assert (isinstance(buf, buffer.Buffer))
	ls = filter(
		(lambda win: (win.data == buf)),
		windows)
	return ls

def getfocus():
	if (focuswin < 0):
		return minibuf.buf
	return windows[focuswin]

def changefocus(self):
	global focuswin

	if (type(self) == type(0)):
		if (self < 0):
			selfpos = -1
			self = minibuf.buf
		else:
			selfpos = self % len(windows)
			self = windows[selfpos]
	else:
		if (self == minibuf.buf):
			selfpos = -1
		else:
			selfpos = windows.index(self)
	
	assert (self == minibuf.buf or self in windows)
	
	if (focuswin < 0):
		cur = minibuf.buf
	else:
		cur = windows[focuswin]
		
	if (self == cur):
		return
	if (self == minibuf.buf and not(minibuf.active)):
		return

	if (focuswin < 0):
		status.statusline.dirty = 1  ### notification via status.watcher?
	else:
		windows[focuswin].modedirty = 1  ###?

	assert (selfpos != focuswin)
	focuswin = selfpos
		
	if (focuswin < 0):
		status.statusline.dirty = 1  ### notification via status.watcher?
	else:
		windows[focuswin].modedirty = 1  ###?
		
# window-specific funcs (could be window methods)

def open(buf):
	global focuswin
	assert (isinstance(buf, buffer.Buffer))
	cla = buf.getviewclass()
	win = cla()
	win.setdata(buf)
	if (focuswin < 0):
		ix = len(windows)
	else:
		ix = focuswin+1
	windows.insert(ix, win)
	recompute_sizes()

def reopen(self, buf):
	assert (self in windows)
	assert (isinstance(buf, buffer.Buffer))
	pos = windows.index(self)
	cla = buf.getviewclass()
	win = cla()
	(vpos, hgt) = (self.vpos, self.height)

	self.setdata(None)
	windows[pos] = win
	win.setdata(buf)
	win.resize(vpos, hgt)

def close(self):
	global focuswin
	assert (self in windows)
	self.setdata(None)
	windows.remove(self)
	if (len(windows) == 0):
		buf = buffer.getbyname('*scratch*', getdefaultbuffer())
		cla = buf.getviewclass()
		win = cla()
		win.setdata(buf)
		windows.append(win)
	if (focuswin >= len(windows)):
		focuswin = len(windows)-1
	recompute_sizes()
	
# late import, to avoid circularity problems
import log
import watcher
import buffer
import minibuf
import status
import execute
import command

execute.registercontext(Window, 'window')

